/*
    (c) 2019 Microchip Technology Inc. and its subsidiaries. 
    
    Subject to your compliance with these terms, you may use Microchip software and any 
    derivatives exclusively with Microchip products. It is your responsibility to comply with third party 
    license terms applicable to your use of third party software (including open source software) that 
    may accompany Microchip software.
    
    THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
    EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY 
    IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS 
    FOR A PARTICULAR PURPOSE.
    
    IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
    INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
    WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP 
    HAS BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO 
    THE FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL 
    CLAIMS IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT 
    OF FEES, IF ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS 
    SOFTWARE.
*/

// Memory Map
//   -----------------
//   |    0x0000     |   Reset vector
//   |               |
//   |    0x0008     |   High Priority Interrupt vector
//   |               |
//   |    0x0018     |   Low Priority Interrupt vector
//   |               |
//   |  Boot Block   |   (this program)
//   |               |
//   |    0x1000     |   Re-mapped Reset Vector (Actual address
//   |    0x1008     |   Re-mapped High Priority Interrupt Vector
//   |    0x1018     |   Re-mapped Low Priority Interrupt Vector
//   |       |       |
//   |               |
//   |  Code Space   |   User program space
//   |               |
//   |       |       |
//   |               |
//   | End of Flash  |
//   -----------------
//
//
//
//
// *****************************************************************************

#define  READ_VERSION   0
#define  READ_FLASH     1
#define  WRITE_FLASH    2
#define  ERASE_FLASH    3
#define  READ_EE_DATA   4
#define  WRITE_EE_DATA  5
#define  READ_CONFIG    6
#define  WRITE_CONFIG   7
#define  CALC_CHECKSUM  8
#define  RESET_DEVICE   9



// *****************************************************************************
    #include <xc.h>       // Standard include
    #include <stdint.h>
    #include <stdbool.h>
    #include "bootload.h"
    #include "mcc.h"
#include "pin_manager.h"

// *****************************************************************************
void     Get_Buffer (void);     // generic comms layer
uint16_t  Get_Version_Data(void);
uint16_t  Read_Flash(void);
uint16_t  Write_Flash(void);
uint16_t  Erase_Flash(void);
uint16_t  Read_EE_Data(void);
uint16_t  Write_EE_Data(void);
uint16_t  Read_Config(void);
uint16_t  Write_Config(void);
uint16_t  Calc_Checksum(void);
void     StartWrite(void);
void     BOOTLOADER_Initialize(void);
void     Run_Bootloader(void);
bool     Bootload_Required (void);

// *****************************************************************************
#define	MINOR_VERSION	0x01       // Version
#define	MAJOR_VERSION	0x01
#define ERROR_ADDRESS_OUT_OF_RANGE   0xFE
#define ERROR_INVALID_COMMAND        0xFF
#define COMMAND_SUCCESS              0x01

// To be device independent, these are set by mcc in memory.h
#define  LAST_WORD_MASK              (WRITE_FLASH_BLOCKSIZE - 1)
#define  NEW_RESET_VECTOR            0x1000
#define  NEW_INTERRUPT_VECTOR_HIGH   0x1008
#define  NEW_INTERRUPT_VECTOR_LOW    0x1018

#define _str(x)  #x
#define str(x)  _str(x)
// *****************************************************************************


// *****************************************************************************
    uint16_t check_sum;      // Checksum accumulator
    uint16_t counter;        // General counter
    uint8_t rx_data;
    uint8_t tx_data;
    bool reset_pending  = false;
// Force variables into Unbanked for 1-cycle accessibility 
    uint8_t EE_Key_1    __at(0x500);
    uint8_t EE_Key_2    __at(0x501);

frame_t  frame;

// *****************************************************************************
// Newer PIC18F devices have vectored interrupts with the base address specified
// by SFR IVTBASE.  Thus there is no need for the bootloader to redirect the 
// Interrupt vectors. The end application should specify IVTBASE as needed
// *****************************************************************************
// *****************************************************************************
void BOOTLOADER_Initialize ()
{
    EE_Key_1 = 0;
    EE_Key_2 = 0;
    BOOTLOADER_INDICATOR = BL_INDICATOR_ON;
    if (Bootload_Required () == true)
    {
        Run_Bootloader ();     // generic comms layer
    }
    STKPTR = 0x00;
	BSR = 0x00;
	BOOTLOADER_INDICATOR = BL_INDICATOR_OFF;
    asm ("goto  "  str(NEW_RESET_VECTOR));
}

// *****************************************************************************
bool Bootload_Required ()
{
    uint8_t u8Data=0;
// ******************************************************************
//  Check an IO pin to force entry into bootloader
// ******************************************************************

    for (uint8_t i = 0; i != 0xFF; i++) NOP();
    u8Data=IO_PIN_ENTRY_PORT_PIN;
    //if(IO_PIN_ENTRY_PORT_PIN==IO_PIN_ENTRY_RUN_BL)//if use as this,  PORT Data abnormal status
    if (u8Data  == IO_PIN_ENTRY_RUN_BL)
    {
        return (true);
    }

// ******************************************************************
//  Check the reset vector for code to initiate bootloader
// ******************************************************************
                                    // This section reads the application start
                                    // vector to see if the location is blank
    TBLPTR = NEW_RESET_VECTOR;      // (0xFF) or not.  If blank, it runs the
                                    // bootloader.  Otherwise, it assumes the
                                    // application is loaded and instead runs the
                                    // application.
    NVMCON1 = 0x80;
    asm("TBLRD *+");
    if (TABLAT == 0xFF)
    {
        return (true);
    }
	return (false);
}

// *****************************************************************************
uint16_t  ProcessBootBuffer()
{
    uint16_t   len;
    EE_Key_1 = frame.EE_key_1;
    EE_Key_2 = frame.EE_key_2;
	
// Test the command field and sub-command.
    switch (frame.command)
    {
    case    READ_VERSION:
        len = Get_Version_Data();
        break;
    case    WRITE_FLASH:
        len = Write_Flash();
        break;
    case    ERASE_FLASH:
        len = Erase_Flash();
        break;
    case    READ_CONFIG:
        len = Read_Config();
        break;
    case    WRITE_CONFIG:
        len = Write_Config();
        break;
    case    CALC_CHECKSUM:
        len = Calc_Checksum();
        break;
    case    RESET_DEVICE:
        frame.data[0] = COMMAND_SUCCESS;
        reset_pending = true;
        len = 10;
        break;
    default:
        frame.data[0] = ERROR_INVALID_COMMAND;
        len = 10;
    }
    return (len);
}

// *****************************************************************************
// Commands
//
//        Cmd     Length----------------   Address---------------
// In:   [<0x00> <0x00><0x00><0x00><0x00> <0x00><0x00><0x00><0x00>]
// OUT:  [<0x00> <0x00><0x00><0x00><0x00> <0x00><0x00><0x00><0x00> <VERL><VERH>]
// *****************************************************************************
uint16_t  Get_Version_Data()
{
    frame.data[0] = MINOR_VERSION;
    frame.data[1] = MAJOR_VERSION;
    frame.data[2] = 0x00;       // Max packet size (256)
    frame.data[3] = 0x01;
    frame.data[4] = 0;
    frame.data[5] = 0;
    TBLPTRU = 0x3F;
    TBLPTRH = 0xFF;               // Get device ID
    TBLPTRL = 0xFE;
    NVMCON1 = 0xC0;       // Setup writes
    asm("TBLRD *+");  //    EECON1bits.RD   = 1;
    frame.data[6] = TABLAT;
    asm("TBLRD *+");   //    EECON1bits.RD   = 1;
    frame.data[7] = TABLAT;
    frame.data[8] = 0;
    frame.data[9] = 0;

    frame.data[10] = ERASE_FLASH_BLOCKSIZE;
    frame.data[11] = WRITE_FLASH_BLOCKSIZE;

    TBLPTRU = 0x30;
    TBLPTRH = 0x00;
    TBLPTRL = 0x00;
//    NVMCON1 = 0xC0;       // Setup writes  Hasn't changed sincelast assignment.
    for (uint8_t  i= 12; i < 16; i++)
    {
        asm("TBLRD *+");
        frame.data[i] = TABLAT;
    }

    return  25;   // total length to send back 9 byte header + 16 byte payload
}



// *****************************************************************************
// Write Flash
//        Cmd     Length----- Keys------   Address---------------  Data ---------
// In:   [<0x02> <0x00><0x00><0x55><0xAA> <0x00><0x00><0x00><0x00> <Data>..<data>]
// OUT:  [<0x02> <0x00><0x00><0x00><0x00> <0x00><0x00><0x00><0x00> <0x01>]
// *****************************************************************************
uint8_t    PFM_Buffer[256] __at(0x3700);
uint16_t Write_Flash()
{
    uint32_t   address;
    
    address = (((uint32_t)frame.address_U) << 16)
            + (((uint32_t)frame.address_H) << 8)
            +   (uint32_t)frame.address_L;
    if (address < NEW_RESET_VECTOR)
    {
        frame.data[0] = ERROR_ADDRESS_OUT_OF_RANGE;
        return (10);
    }
    FLASH_ReadPage(address);
  
    for (uint16_t  i = 0; i < frame.data_length; i ++)
    {
        PFM_Buffer [frame.address_L + i] = frame.data[i];
    }
    //Set NVMADR with the target word address
    NVMADRU = frame.address_U;
    NVMADRH = frame.address_H;
    NVMADRL = frame.address_L;

    //Set the NVMCMD control bits for Write Page operation
    NVMCON1bits.NVMCMD = 0b101;

    //Disable all interrupt
    INTCON0bits.GIE = 0;

    //Perform the unlock sequence
    StartWrite();
    FLASH_WriteBlock(address,PFM_Buffer);
    //Start page programming and wait for the operation to complete
    NVMCON0bits.GO = 1;
    while (NVMCON0bits.GO);

    frame.data[0] = COMMAND_SUCCESS;
    EE_Key_1 = 0x00;  // erase EE Keys
    EE_Key_2 = 0x00;
    return (10);
}


// *****************************************************************************
// Erase Program Memory
// Erases data_length rows from program memory
//        Cmd     Length----- Keys------   Address---------------  Data ---------
// In:   [<0x03> <0x00><0x00><0x55><0xAA> <0x00><0x00><0x00><0x00>]
// OUT:  [<0x03> <0x00><0x00><0x00><0x00> <0x00><0x00><0x00><0x00> <0x01>]
// *****************************************************************************
uint16_t Erase_Flash ()
{
    uint32_t   address;
    
    TBLPTRL = frame.address_L;
    TBLPTRH = frame.address_H;
    TBLPTRU = frame.address_U;
    address = (((uint32_t)frame.address_U) << 16)
            + (((uint32_t)frame.address_H) << 8)
            +   (uint32_t)frame.address_L;

    uint8_t GIEBitValue = INTCON0bits.GIE;

    //Disable all interrupts
    INTCON0bits.GIE = 0;

    for (uint16_t i=0; i < frame.data_length; i++)
    {
        uint32_t blockStartAddr = (uint32_t) (address & ((END_FLASH - 1) ^ ((ERASE_FLASH_BLOCKSIZE * 2) - 1)));

        //The NVMADR[21:8] bits point to the page being erased.
        //The NVMADR[7:0] bits are ignored
        NVMADRU = (uint8_t) ((blockStartAddr & 0x00FF0000) >> 16);
        NVMADRH = (uint8_t) ((blockStartAddr & 0x0000FF00) >> 8);

        //Set the NVMCMD control bits for Erase Page operation
        NVMCON1bits.NVMCMD = 0b110;

        //Perform the unlock sequence
        StartWrite();
        address += ERASE_FLASH_BLOCKSIZE * 2;
    }
    //Restore the interrupts
    INTCON0bits.GIE = GIEBitValue;

    //Set the NVMCMD control bits for Word Read operation to avoid accidental writes
    NVMCON1bits.NVMCMD = 0b000;
    frame.data[0]  = COMMAND_SUCCESS;
    frame.EE_key_1 = 0x00;  // erase EE Keys
    frame.EE_key_2 = 0x00;
    return (10);
}


// *****************************************************************************
// Read Config Words
//        Cmd     Length-----              Address---------------  Data ---------
// In:   [<0x06> <0x00><0x00><0x00><0x00> <0x00><0x00><0x00><0x00>]
// OUT:  [<0x06> <0x00><0x00><0x00><0x00> <0x00><0x00><0x00><0x00> <Data>..<data>]
// *****************************************************************************
uint16_t Read_Config ()
{
    TBLPTRL = frame.address_L;
    TBLPTRH = frame.address_H;
    TBLPTRU = frame.address_U;
	NVMCON1   = 0xC0;
    for (uint8_t  i = 0; i < frame.data_length; i++)
    {
        asm("TBLRD *+");
        frame.data[i] = TABLAT;
    }
    return (9 + frame.data_length);           // 9 byte header + 4 bytes config words
}

// *****************************************************************************
// Write Config Words
//        Cmd     Length-----              Address---------------  Data ---------
// In:   [<0x07> <0x00><0x00><0x55><0xAA> <0x00><0x00><0x00><0x00> <Data>..<data>]
// OUT:  [<0x07> <0x00><0x00><0x00><0x00> <0x00><0x00><0x00><0x00>]
// *****************************************************************************
uint16_t Write_Config ()
{
    TBLPTRL = frame.address_L;
    TBLPTRH = frame.address_H;
    TBLPTRU = frame.address_U;
    if (TBLPTR > 0x20000F)    
        NVMCON1 = 0xC4;       // Setup writes
    else
        NVMCON1 = 0x84;
    for (uint8_t  i= 0; i < frame.data_length; i ++)
    {
        TABLAT = frame.data[i];
        asm("TBLWT *");
        StartWrite();
        ++ TBLPTRL;
    }
    frame.data[0] = COMMAND_SUCCESS;
    EE_Key_1 = 0x00;  // erase EE Keys
    EE_Key_2 = 0x00;
    return (10);
}

// *****************************************************************************************
// Calculate Checksum
// In:	[<0x08><DataLengthL><DataLengthH> <unused><unused> <ADDRL><ADDRH><ADDRU><unused>...]
// OUT:	[9 byte header + ChecksumL + ChecksumH]
//        Cmd     Length-----              Address---------------  Data ---------
// In:   [<0x08> <0x00><0x00><0x00><0x00> <0x00><0x00><0x00><0x00>]
// OUT:  [<0x08> <0x00><0x00><0x00><0x00> <0x00><0x00><0x00><0x00> <CheckSumL><CheckSumH>]
// *****************************************************************************
uint16_t Calc_Checksum()
{
#if END_FLASH > 0x10000
    uint32_t  i;
    uint32_t  length;
#else
    uint16_t  i;
    uint16_t  length;
#endif 

    TBLPTRL = frame.address_L;
    TBLPTRH = frame.address_H;
    TBLPTRU = frame.address_U;
    NVMCON1 = 0x80;
    check_sum = 0;
    length = frame.data_length;
#if END_FLASH > 0x10000
    length += ((uint32_t) frame.EE_key_1) << 16;
#endif 
    for (i = 0; i < length; i += 2)
    {
        asm("TBLRD *+");
        check_sum += (uint16_t)TABLAT;
        asm("TBLRD *+");
        check_sum += (uint16_t)TABLAT << 8;
     }
     frame.data[0] = (uint8_t) (check_sum & 0x00FF);
     frame.data[1] = (uint8_t)((check_sum & 0xFF00) >> 8);
     return (11);
}




// *****************************************************************************
// Unlock and start the write or erase sequence.
// *****************************************************************************
void StartWrite()
{
    __asm ("banksel NVMLOCK");
    __asm ("movf " ___mkstr(BANKMASK(_EE_Key_1)) ", w, c");
    __asm ("movwf NVMLOCK, b");
    __asm ("movf " ___mkstr(BANKMASK(_EE_Key_2)) ", w, c");
    __asm ("movwf NVMLOCK, b");
   __asm ("bsf   NVMCON0,0");       // Start the write (bit 0)
    NOP();
    NOP();
    return;
}

// *****************************************************************************
// Check to see if a device reset had been requested.  We can't just reset when
// the reset command is issued.  Instead we have to wait until the acknowledgement
// is finished sending back to the host.  Then we reset the device.
// *****************************************************************************
void Check_Device_Reset ()
{
    if (reset_pending == true)
    {
        BOOTLOADER_INDICATOR = BL_INDICATOR_OFF;
        RESET();
    }
    return;
}

// *****************************************************************************
// *****************************************************************************
